import re
from gallery_utils import *

def title(source):
    gfyId = re.findall(r'href=\".*gfycat.com/(\w+).*\">', source)[-1]
    link = 'https://gfycat.com/cajax/get/' + gfyId
    respond = urlopen_text(link)
    username = re.findall(r'\"userName\":\"(.+?)\",' ,respond)[0]
    return username if username != "anonymous" else "gfycat " + gfyId

def redirect(source):
    gfyId = re.findall(r'href=\".*gfycat.com/(\w+).*\">', source)[-1]
    respond = urlopen_text('https://gfycat.com/cajax/get/' + gfyId)
    webmurl = re.findall(r'\"webmUrl\":\"(.+?)\",' ,respond)[0]
    # delete escape characters
    webmurl = webmurl.replace("\\","")
    # for some reason we can not connect via https
    webmurl = webmurl.replace("https", "http")
    return webmurl
same_filename = True



# Initialization file for gallery_get
#
# Rego Sen
# Aug 22, 2013
#

DEFAULT_TITLE = r'<title>(.*?)</title>'
DEFAULT_REDIRECT = "" # assumes all links are direct links
DEFAULT_DIRECT_LINKS = r'src=[\"\'](.+?\.jpe?g)[\"\']'
DEFAULT_USE_FILENAME = False

import os,sys
PLUGINS = {}
DEFAULT_PLUGIN = None
FALLBACK_TITLE = "Untitled Gallery"

class Plugin(object):
    def __init__(self, debugname, identifier):
        self.debugname = debugname
        self.identifier = identifier
        self.redirect = DEFAULT_REDIRECT
        self.direct = DEFAULT_DIRECT_LINKS
        self.title = DEFAULT_TITLE
        self.useFilename = DEFAULT_USE_FILENAME

def register_plugin(modname, debugname, identifier):
    if not modname in PLUGINS:
        PLUGINS[modname] = Plugin(debugname, identifier)

def register_title(modname, title):
    PLUGINS[modname].title = title
    
def register_redirect(modname, redirect):
    PLUGINS[modname].redirect = redirect
    
def register_links(modname, direct):
    PLUGINS[modname].direct = direct
    
def register_usefile(modname, useFile):
    PLUGINS[modname].useFilename = useFile

# import all python files starting with "plugin_"
directory = os.path.abspath(os.path.dirname(__file__))
sys.path.insert(0, directory)
plugin_prefix = "plugin_"
for file in os.listdir(directory):
    filelower = file.lower()
    if filelower.startswith(plugin_prefix) and filelower.endswith(".py"):
        f, e = os.path.splitext(file)
        try:
            mod = __import__(f)
            locals = dir(mod)
            name = mod.__name__
            debugname = name[len(plugin_prefix):] # text after prefix
            if not 'identifier' in locals:
                mod.identifier = debugname
            register_plugin(name, debugname, mod.identifier)
            if 'title' in locals: register_title(name, mod.title)
            if 'redirect' in locals: register_redirect(name, mod.redirect)
            if 'direct_links' in locals: register_links(name, mod.direct_links)
            if 'same_filename' in locals: register_usefile(name, mod.same_filename)
        except ImportError:
            print("Gallery: failed to import")
            print(f, ":", sys.exc_value)



# Plugin for gallery_get.
import re

# Each definition can be one of the following:
# - a string
# - a regex string
# - a function that takes source as a parameter and returns an array or a string.  (You may assume that re and urllib are already imported.)
# If you comment out a parameter, it will use the default defined in __init__.py

# identifier (default = name of this plugin after "plugin_") : If there's a match, we'll attempt to download images using this plugin.

# title: parses the gallery page for a title.  This will be the folder name of the output gallery.
title = r'<title>(.*?) - xHamster.com</title>'

# redirect: if the links in the gallery page go to an html instead of an image, use this to parse the gallery page.
def redirect(source):
    redirects = re.findall(r'property="og:url" content="(\S+?)"', source)
    indexed_source = source
    while True:
        next_url = re.findall(r'rel="next" href="(\S+?)"', indexed_source)
        if next_url:
          indexed_page = next_url[0]
          redirects.append(indexed_page)
          print("Crawling " + indexed_page)
          try:
            indexed_source = urlopen_text(indexed_page)
          except:
            break
        else:
          break
    return redirects

# direct_links: if redirect is non-empty, this parses each redirect page for a single image.  Otherwise, this parses the gallery page for all images.
def direct_links(source):
    links = re.findall(r'"imageURL":"(\S+?)"',source)
    return map(lambda x: x.replace("\/", "/"), links)

# same_filename (default=False): if True, uses filename specified on remote link.  Otherwise, creates own filename with incremental index. 
same_filename = True


# REDDIT_GET is a tool for downloading all imgur albums and pictures
# that were submitted by a given reddit user.
#
# DEPENDENCIES
# This relies on gallery_get and the imgur_album plugin.
#
# See gallery_get for more info
#
# Rego Sen
# Nov 2, 2013
#

import gallery_get
import os, time, sys, traceback
import datetime, json

from gallery_utils import *
import gallery_plugins

USER_QUERY = "http://www.reddit.com/user/%s/submitted/.json?limit=1000"
DEST_ROOT = gallery_get.DEST_ROOT
safe_makedirs(DEST_ROOT)

# To speed this up, don't craw links with pages that we know aren't galleries
NON_GALLERY_DOMAINS = [
"youtube.com",
"youtu.be",
"www.reddit.com"
]

def is_individual_imgur(url):
    if "/imgur.com/a/" in url:
        return False
    if "/imgur.com/gallery/" in url:
        return False
    return True

class RedditGet(object):
    def __init__(self, user, dest, flush_jobs=True):
        self.user = user
        self.dest = dest
        self.flush_jobs = flush_jobs

    def get_user_json(self):
        reddit_json_str = ""
        reddit_json = {}
        cache_path = self.user + ".json"
        query_url = USER_QUERY % self.user
        if os.path.exists(cache_path):
            print("Getting JSON data from local file (%s)" % cache_path)
            reddit_json_str = unicode_safe(open(cache_path,"r").read())
            reddit_json = json.loads(reddit_json_str)
        else:
            print("Requesting JSON data from reddit...")
            for i in range(5):
                try:
                    reddit_json_str = urlopen_text(query_url)
                    reddit_json = json.loads(reddit_json_str)
                except URLError:
                    break
                except Exception as e:
                    if hasattr(e, 'code') and e.code == 404:
                        break
                if "data" in reddit_json:
                    break
                else:
                    time.sleep(2) # workaround for server-side request frequency issues

        if not "data" in reddit_json:
            print("ERROR getting json data after several retries!  Does the user exist?")
            print("If so, try saving the contents of the following to %s and try again." % cache_path)
            print(query_url)
        return reddit_json

    def folder_from_post(self, data):
        sdate = datetime.datetime.fromtimestamp(data['created']).strftime("%Y-%m-%d")
        title = data['title'].replace('/', '_').replace('\\', '_').strip()
        if title:
            title = " - " + title

        return os.path.join(unicode_safe(self.dest), self.user, gallery_get.safe_str(sdate + title))

    # includes special shortcuts for skipping the redirect
    def process_reddit_post(self, url, folder):
        if "/i.reddituploads.com/" in url:
            gallery_get.download_image(url + ".jpg", folder)
        elif "/imgur.com/" in url and is_individual_imgur(url):
            # Create direct image URL with dummy extension (otherwise it will redirect)
            # Then get correct extension from header
            img_base = url.replace("/imgur.com/","/i.imgur.com/")
            ext = "jpg"
            file = urlopen_safe("%s.%s" % (img_base, ext))
            real_ext = file.headers.get("content-type")[6:]
            if real_ext != "jpeg": # jpeg -> jpg
                ext = real_ext
            gallery_get.download_image("%s.%s" % (img_base, ext), folder)
        elif "/i.imgur.com/" in url:
            gallery_get.download_image(url, folder)
        else:
            # TODO: use Queue or eventlet for launching gallery_get.run_wrapped()
            gallery_get.run_wrapped(url, folder, titleAsFolder=True, cacheDest=False, flushJobs=False, allowGenericPlugin=False)

    def run(self):
        reddit_json = self.get_user_json()

        if "data" in reddit_json:
            visited_links = set()
            num_valid_posts = 0
            for post in reddit_json['data']['children']:
                data = post['data']
                url = data['url']
                domain = urlparse(url).netloc.lower()
                if any(x in domain for x in NON_GALLERY_DOMAINS):
                    print("Skipping non-gallery link: " + url)
                    continue
                elif url.lower() in visited_links:
                    print("Skipping already visited link: " + url)
                    continue
                else:
                    visited_links.add(url.lower())

                self.process_reddit_post(url, self.folder_from_post(data))
        if self.flush_jobs:
            gallery_get.flush_jobs()

def run_wrapped(user, dest="", flush_jobs=True):
    global DEST_ROOT
    try:
        if dest:
            gallery_get.safeCacheDestination(dest)
            DEST_ROOT = unicode_safe(dest)
        RedditGet(user, dest or DEST_ROOT, flush_jobs).run()
    except:
        print('\n' + '-'*60)
        traceback.print_exc(file=sys.stdout)
        print("Using params: [%s, %s]" % (user, dest))
        print('-'*60 + '\n')
        print(gallery_get.EXCEPTION_NOTICE)
    return os.path.join(DEST_ROOT, user)

def run_prompted():
    user = str_input("Input reddit user: ").strip()
    if not user:
        print("Nothing to do!")
        sys.exit()
    dest = str_input("Destination (%s): " % encode_safe(DEST_ROOT)).strip()
    if dest:
        gallery_get.safeCacheDestination(dest)
    RedditGet(user, dest or DEST_ROOT).run()

def run(user="", dest=""):
    if not user:
        run_prompted()
    else:
        run_wrapped(user, dest)

cur_file = os.path.basename(str(__file__))
arg_file = sys.argv[0]
if arg_file and os.path.basename(arg_file) == cur_file:
    ### DIRECT LAUNCH (not import)
    if len(sys.argv) > 1:
        # use first parameter as reddit user, second (if exists) as dest
        if len(sys.argv) > 2:
            run_wrapped(sys.argv[1], sys.argv[2])
        else:
            run_wrapped(sys.argv[1])
    else:
        run_prompted()



# Plugin for gallery_get.
import re
from gallery_utils import *

# Each definition can be one of the following:
# - a string
# - a regex string
# - a function that takes source as a parameter and returns an array or a string.  (You may assume that re and urllib are already imported.)
# If you comment out a parameter, it will use the default defined in __init__.py

# identifier (default = name of this plugin after "plugin_") : If there's a match, we'll attempt to download images using this plugin.

# title: parses the gallery page for a title.  This will be the folder name of the output gallery.

# redirect: if the links in the gallery page go to an html instead of an image, use this to parse the gallery page.
def redirect(source):
    redirects = []
    base_url = "http://shimmie.shishnet.org"
    cur_url = re.findall(r'href="(.+?)"\>Random</a>', source)[0].rsplit("/",1)[0]
    index = 0
    while True:
        indexed_page = "%s%s/%s" % (base_url, cur_url, index)
        print("Crawling " + indexed_page)
        try:
          indexed_source = urlopen_text(indexed_page)
          links = re.findall("href='(.+?)' class='thumb", indexed_source)
          if links:
              redirects += map(lambda x: base_url + x, links)
              index += 1
          else:
              break
        except:
          break
    return redirects

# direct_links: if redirect is non-empty, this parses each redirect page for a single image.  Otherwise, this parses the gallery page for all images.
direct_links = r"name='ubb_full-img' value='\[img\](.+?)\[/img\]'"


# same_filename (default=False): if True, uses filename specified on remote link.  Otherwise, creates own filename with incremental index. 
same_filename = True


import os, re, csv
import gallery_get, reddit_get
from gallery_utils import *
from optparse import OptionParser

DEST_ROOT = gallery_get.DEST_ROOT
INPUT_PATH = "gallery_get_test_input.csv"
OUTPUT_PATH = "gallery_get_test_output.csv"
FAILED_TEST_CASES = []

parser = OptionParser()
parser.add_option(
        '--noprompt', dest='noprompt', action='store_true', default=False,
        help='don\'t prompt for extra galleries')
parser.add_option(
        '--input', dest='input_path', default=INPUT_PATH,
        help='input CSV for testing (same format as output)')
(options, args) = parser.parse_args()


### TESTING CLASSES

class GalleryTest(object):
  def __init__(self, debugname):
    self.debugname = debugname
    self.title = ""
    self.files = []
    self.size = 0
    self.status = None
    print("")
    print("Testing " + debugname)

  def post_run(self, root, success, params):
    
    if root:
      self.title = encode_safe(os.path.basename(root))
      for (dirpath, dirnames, filenames) in os.walk(root):
        self.files += filenames
        for filename in filenames:
          self.size += os.path.getsize(os.path.join(dirpath,filename))
    self.status = True if (success and root and self.size) else False
    if not self.status:
      FAILED_TEST_CASES.append("DOWNLOADS FAILED in %s: Please check messages above." % self.debugname)
    if params:
      self.compare(*params)


  def _compare_internal(self, real_val, expected_val, name, allow_greater=False):
    global FAILED_TEST_CASES
    if expected_val and (real_val != expected_val):
      if allow_greater and (real_val > expected_val):
        FAILED_TEST_CASES.append("TEST NEEDS UPDATE for %s: %s = %s (expected %s)" % (self.debugname, name, str(real_val), str(expected_val)))
      else:
        FAILED_TEST_CASES.append("TEST FAILED in %s: %s = %s (expected %s)" % (self.debugname, name, str(real_val), str(expected_val)))
        self.status = False

  def compare(self, title="", num_files=0, size=0):
    global FAILED_TEST_CASES
    self._compare_internal(self.title, title, "title")
    self._compare_internal(len(self.files), int(num_files), "number of files", True)
    self._compare_internal(self.size, int(size), "total size", True)
    return self.status


class GalleryGetTest(GalleryTest):
  def __init__(self, params):
    self.url = params[0]
    GalleryTest.__init__(self, self.url)

    root = gallery_get.run_wrapped(self.url, "", flushJobs=False)
    success = gallery_get.flush_jobs()
    self.post_run(root, success, params[1:])


class RedditGetTest(GalleryTest):
  def __init__(self, params):
    self.user = params[0]
    GalleryTest.__init__(self, self.user)

    root = reddit_get.run_wrapped(self.user, "", False)
    success = gallery_get.flush_jobs()
    self.post_run(root, success, params[1:])


### REPORTING METHODS

def read_input(input_path):
  rows = []
  if os.path.exists(input_path):
    report_file = open(input_path,"r")
    reader = csv.reader(report_file)
    rows = list(reader)[1:]
    report_file.close()
  return rows


def write_report(results, output_path):
  if not results:
    print("Nothing to test!")
    return

  report = ["URL/User,Title,Files,Size"]
  for result in results:
    report.append("%s,%s,%d,%d" % (result.debugname, result.title, len(result.files), result.size))

  file = open(output_path,"w")
  file.write("\n".join(report))
  file.close()

  print("")
  if FAILED_TEST_CASES:
    print("="*40)
    print("\n".join(FAILED_TEST_CASES))
    print("="*40)

  succeeded = sum(test.status == True for test in results)
  failed = sum(test.status == False for test in results)
  pending = sum(test.status == None for test in results)
  if pending > 0:
    print("Summary: %d succeeded, %d failed, %d ready to add" % (succeeded, failed, pending))
  else:
    print("Summary: %d succeeded, %d failed" % (succeeded, failed))
  print("Results written to %s" % output_path)
  print("")


### RUN TESTS AND WRITE REPORT

tests = read_input(options.input_path)
if not options.noprompt:
  while True:
    extra_test = str_input("Extra URL or Reddit user to test (blank for none): ")
    if extra_test:
      tests.append([extra_test])
    else:
      break

results = []
for row in tests:
  if re.match("\A[A-Za-z0-9_]+\Z", row[0]):
    results.append(RedditGetTest(row))
  else:
    results.append(GalleryGetTest(row))

write_report(results, OUTPUT_PATH)




# Plugin for gallery_get.
import re

# Each definition can be one of the following:
# - a string to match
# - a regex string to match
# - a function that takes source as a parameter and returns an array or a single match.  (You may assume that re and urllib are already imported.)
# If you comment out a parameter, it will use the default defined in __init__.py

# identifier (default = name of this plugin after "plugin_") : If there's a match, we'll attempt to download images using this plugin.
identifier = "www.vidble.com"

# title: parses the gallery page for a title.  This will be the folder name of the output gallery.
title = r'<title>"(.*?)</title>"'

# redirect: if the links in the gallery page go to an html instead of an image, use this to parse the gallery page.

# direct_links: if redirect is non-empty, this parses each redirect page for a single image.  Otherwise, this parses the gallery page for all images.
# * if using regex, you can have two matches: the first will be the link and the second will be the basename of the file.
#   if the matches need to be reversed, use named groups "link" and "basename"

def direct_links(source):
    links = re.findall('img src=[\"\'](.+?\.jpe?g)[\"\']',source)
    links = filter(lambda x: "/assets/" not in x, links)
    return map(lambda x: 'http://vidble.com' + x, links)

# same_filename (default=False): if True, uses same filename from remote link.  Otherwise, creates own filename with incremental index (or uses subtitle). 


from distutils.core import setup
import glob

mods = glob.glob("*.py") + glob.glob("gallery_plugins/*.py") # not using recursive "**", unsupported in Python 2
mods.remove("setup.py")
setup(name = 'gallery_get',
    version = '1.7.11',
    author = 'Rego Sen',
    author_email = 'regosen@gmail.com',
    url = 'https://github.com/regosen/gallery_get',
    description = 'Gallery downloader - supports many galleries and reddit user histories',
    long_description = open("README.rst","r").read(),
    license = 'MIT',
    keywords = 'gallery downloader reddit imgur 4chan xhamster eroshare vidble pornhub xvideos',
    py_modules = [x.replace(".py","") for x in mods],
    packages=['gallery_plugins'],
    classifiers = [
        'Development Status :: 4 - Beta',
        'Environment :: Console',
        'Intended Audience :: End Users/Desktop',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 3',
        'Topic :: Internet',
        'Topic :: Multimedia :: Graphics',
    ],
)


import os

# Python 3 imports that throw in Python 2
try:
    from urllib.request import Request, urlopen, URLError
    from urllib.parse import urlparse, unquote
except ImportError:
    # This is Python 2
    from urllib2 import Request, urlopen, URLError
    from urlparse import urlparse
    from urllib import unquote

# Python 2 types that throw in Python 3
try:
    str_input = raw_input
    str_type = basestring
except:
    # This is Python 3
    str_input = input
    str_type = str


# some galleries reject requests if they're not coming from a browser- this is to get past that.
def urlopen_safe(url):
    q = Request(url)
    q.add_header('User-Agent', 'Mozilla/5.0')
    return urlopen(q)

def safe_makedirs(folder):
    if not os.path.exists(folder):
        os.makedirs(folder)


# Python 2<>3 compatibility methods
def encode_safe(in_str):
    try:
        if isinstance(in_str,unicode):
            in_str = in_str.encode("utf8")
    except:
        pass
    return in_str

def unicode_safe(str):
    try:
        str = str.decode("utf8")
    except:
        pass
    return str

def urlopen_text(url):
    return unicode_safe(urlopen_safe(url).read())
    
def is_str(obj):
    return isinstance(obj, str_type)

def is_iterable(obj):
    return hasattr(obj, '__iter__') and not is_str(obj)

